use num_format::{Locale, ToFormattedString};
use serenity::prelude::*;
use serenity::{
    framework::standard::{macros::command, Args, CommandResult},
    model::channel::Message,
};

#[command]
#[description = "echos the message given"]
pub async fn profile(ctx: &Context, msg: &Message, args: Args) -> CommandResult {
    let rsn = args.rest().replace(" ", "%20");
    let resp = reqwest::get(&format!(
        "https://apps.runescape.com/runemetrics/profile/profile?user={}&activities=20",
        rsn
    ))
    .await?;
    let profile: Profile = match resp.json::<Profile>().await {
        Ok(p) => p,
        Err(why) => {
            println!("{}", why);
            return Ok(());
        }
    };
    let quest_total = profile.questscomplete + profile.questsstarted + profile.questsnotstarted;
    let right = format!(
        "**Quests:** {}/{} ({} in progress)
**Rank:** {}
**Total level:** {}
**Total XP:** {}
**Combat level:** {}",
        profile.questscomplete,
        quest_total,
        profile.questsstarted,
        profile.rank.clone().unwrap_or("none".to_string()),
        profile.totalskill,
        profile.totalxp.to_formatted_string(&Locale::en),
        profile.combatlevel
    );
    let mut stats_extended: Vec<SkillValueExtended> = profile
        .skillvalues
        .iter()
        .map(|s| SkillValueExtended::from(s))
        .collect();
    stats_extended.sort_by(|s1, s2| ORDER[s1.id as usize].cmp(&ORDER[s2.id as usize]));
    let mut left = String::new();
    for i in 0..28 {
        if i % 3 == 0 {
            left.push('\n')
        }
        let stat = &stats_extended[i];
        left.push_str(&format!("{}{}", EMOJIS[stat.id as usize], stat.level));
    }
    msg.channel_id
        .send_message(&ctx.http, |m| {
            m.embed(|e| {
                e.author(|a| a.name(format!("RuneMetrics profile of: {}", profile.name)))
                    .field("Levels", left, true)
                    .field("Info", right, true)
            })
        })
        .await?;
    let mut activities_out = String::new();
    for (i, a) in profile.activities.iter().enumerate() {
        {
            activities_out.push_str(&format!("**{}** {}\n```{}```", a.date, a.text, a.details));
        }
        if i == 10 {
            msg.channel_id.say(&ctx.http, &activities_out).await?;
            activities_out.clear();
        }
    }
    if !activities_out.is_empty() {
        msg.channel_id.say(&ctx.http, &activities_out).await?;
    }
    Ok(())
}

use serde::Deserialize;

#[derive(Deserialize, Debug)]
struct Activity {
    date: String,
    details: String,
    text: String,
}

#[derive(Deserialize, Debug)]
struct Profile {
    magic: u32,
    questsstarted: u32,
    totalskill: u32,
    questscomplete: u32,
    questsnotstarted: u32,
    totalxp: u64,
    ranged: u32,
    activities: Vec<Activity>,
    skillvalues: Vec<SkillValue>,
    name: String,
    rank: Option<String>,
    melee: u32,
    combatlevel: u32,
    #[serde(alias = "loggedIn")]
    logged_in: String,
}

#[derive(Deserialize, Debug)]
struct SkillValue {
    level: u32,
    xp: u32,
    rank: Option<u32>,
    id: u32,
}

#[derive(Debug)]
struct SkillValueExtended {
    level: u32,
    xp: u32,
    rank: Option<u32>,
    id: u32,
    name: String,
}

impl std::convert::From<&SkillValue> for SkillValueExtended {
    fn from(sv: &SkillValue) -> Self {
        Self {
            name: SKILL_NAMES[sv.id as usize].to_string(),
            level: sv.level,
            xp: sv.xp,
            rank: sv.rank,
            id: sv.id,
        }
    }
}

static SKILL_NAMES: [&'static str; 28] = [
    "Attack",
    "Defence",
    "Strength",
    "Constitution",
    "Ranged",
    "Prayer",
    "Magic",
    "Cooking",
    "Woodcutting",
    "Fletching",
    "Fishing",
    "Firemaking",
    "Crafting",
    "Smithing",
    "Mining",
    "Herblore",
    "Agility",
    "Thieving",
    "Slayer",
    "Farming",
    "Runecrafting",
    "Hunter",
    "Construction",
    "Summoning",
    "Dungeoneering",
    "Divination",
    "Invention",
    "Archeology",
];

static ORDER: [u32; 28] = [
    1, 7, 4, 2, 10, 13, 16, 12, 18, 17, 9, 15, 14, 6, 3, 8, 5, 11, 20, 21, 19, 23, 22, 24, 25, 26,
    27, 28,
];

static EMOJIS: [&'static str; 28] = [
    "<:Attack:406361343223136256>",
    "<:Defence:406361343348834304>",
    "<:Strength:406361343357222914>",
    "<:Constitution:406361343222874112>",
    "<:Ranged:406361343298502658>",
    "<:Prayer:406361343445434390>",
    "<:Magic:406361343608881152>",
    "<:Cooking:406361343361548298>",
    "<:Woodcutting:406361343718064128>",
    "<:Fletching:406361343353159691>",
    "<:Fishing:406361343583846410>",
    "<:Firemaking:406361343718064129>",
    "<:Crafting:406361343168610305>",
    "<:Smithing:406361343487115265>",
    "<:Mining:406361343583584256>",
    "<:Herblore:406361343554355210>",
    "<:Agility:406361343210553344>",
    "<:Thieving:406361343302696962>",
    "<:Slayer:406361343407685633>",
    "<:Farming:406361343407423498>",
    "<:Runecrafting:406361343596298250>",
    "<:Hunter:406361343474532353>",
    "<:Construction:406361343302565888>",
    "<:Summoning:406361343843631107>",
    "<:Dungeoneering:406361343386451979>",
    "<:Divination:406361343374131211>",
    "<:Invention:406361343591972864>",
    "<:Archeology:713541764874764289>",
];
